/* Initializes CPU and basic hardware such as memory 
 * controllers, IRQ controller and system timer 0.
 *
 */

#include <common.h>
#include <asm/asi.h>
#include <asm/leon.h>
#include <asm/leon3.h>
#include <ambapp.h>

#include <config.h>

DECLARE_GLOBAL_DATA_PTR;

/* reset CPU (jump to 0, without reset) */
void start(void);

/* find & initialize the memory controller */
int init_memory_ctrl(void);

LEON3_IrqCtrl_Regs_Map *irqmp = NULL;
LEON3_MEMCTRL memctrl;
LEON3_GpTimer_Regs_Map *gptimer = NULL;
unsigned int gptimer_irq = 0;
LEON3_APBUART_Regs_Map *leon3_apbuart = NULL;

struct {
	gd_t gd_area;
	bd_t bd;
} global_data;

/*
 * Breath some life into the CPU...
 *
 * Set up the memory map,
 * initialize a bunch of registers.
 *
 * Run from FLASH/PROM:
 *  - until memory controller is set up, only registers avaiable
 *  - no global variables available for writing
 *  - constants avaiable
 */

void cpu_init_f (void)
{
	/* these varaiable must not be initialized */
	LEON3_IrqCtrl_Regs_Map *irqmp;
	ambapp_apbdev apbdev;
	register unsigned int apbmst;
	 
	/* find AMBA APB Master */
	apbmst = ambapp_ahb_next_nomem(VENDOR_GAISLER,GAISLER_APBMST,1,0);
	if ( !apbmst ) {
		/* no AHB/APB bridge, something is wrong ==> jump to start (or hang)  */
		while(1) ;
	}
	/* Init memory controller */	
	if ( init_memory_ctrl() ){
		while(1) ;
	}
	
	/****************************************************
	 * From here we can use the main memory and the stack.
	 */
	
	/* Find AMBA APB IRQMP Controller */
	if ( ambapp_apb_first(VENDOR_GAISLER,GAISLER_IRQMP,&apbdev) != 1) {
		/* no IRQ controller, something is wrong ==> jump to start (or hang)  */
		while(1) ;
	}
	irqmp = (LEON3_IrqCtrl_Regs_Map *)apbdev.address;

	/* initialize the IRQMP */
	irqmp->ilevel = 0xf; /* all IRQ off */
	irqmp->iforce = 0;
	irqmp->ipend = 0;
	irqmp->iclear = 0xfffe; /* clear all old pending interrupts */
	
	/* cache */
}

/*
 * initialize higher level parts of CPU like time base and timers
 */
int cpu_init_r (void)
{
	ambapp_apbdev apbdev;
	
	/* Find AMBA APB IRQMP Controller, When we come so far we know there is a IRQMP available */
	ambapp_apb_first(VENDOR_GAISLER,GAISLER_IRQMP,&apbdev);
	irqmp = (LEON3_IrqCtrl_Regs_Map *)apbdev.address;
	
	/* timer */
	if ( ambapp_apb_first(VENDOR_GAISLER,GAISLER_GPTIMER,&apbdev) != 1 ){
		printf("cpu_init_r: gptimer not found!\n");
		return 1;
	}
	gptimer = (LEON3_GpTimer_Regs_Map *)apbdev.address;
	gptimer_irq = apbdev.irq;
	
	/* initialize prescaler common to all timers to 1MHz */
	gptimer->scalar = gptimer->scalar_reload = (((CONFIG_SYS_CLK_FREQ/1000)+500)/1000) - 1;
	
	return (0);
}

/* find & setup memory controller */
int init_memory_ctrl(){
	register LEON3_MCTRL *mctrl;
	register LEON3_SDCTRL *sdctrl;
	register unsigned int base;
	
	/* find ESA Memory controller */
	base = ambapp_apb_next_nomem(VENDOR_ESA,ESA_MCTRL,0);
	if ( base ){
		mctrl = (LEON3_MCTRL *)base;

		/* config MCTRL memory controller */
		mctrl->mcfg1 = CFG_GRLIB_MEMCFG1 | (mctrl->mcfg1 & 0x300);
		mctrl->mcfg2 = CFG_GRLIB_MEMCFG2;
		mctrl->mcfg3 = CFG_GRLIB_MEMCFG3;
		//memctrl.ctrltype = LEON_MEMCTRL_MCTRL;
		//memctrl.mbase.mctrl = mctrl;
		return 0;
	}
	
	/* find SDRAM controller */
	base = ambapp_apb_next_nomem(VENDOR_GAISLER,GAISLER_SDCTRL,0);
	if ( base ){
		sdctrl = (LEON3_SDCTRL *)base;
		
		/* config memory controller */
		sdctrl->sdcfg = CFG_GRLIB_SDRAM;
		//memctrl.ctrltype = LEON_MEMCTRL_SDCTRL;
		//memctrl.mbase.sdctrl = sdctrl;
		return 0;
	}
	
	/* failed to find any memory controller, assuming SRAM/PROM */
	//memctrl.ctrltype = LEON_MEMCTRL_SRAM;
	return -1;
}

/* Uses Timer 0 to get accurate
 * pauses. Max 2 raised to 32 ticks
 *
 */
void cpu_wait_ticks(unsigned long ticks){
	unsigned long start = get_timer(0);
	while ( get_timer(start) < ticks )
		;

#if 0
	LEON3_GpTimerElem_Regs_Map *timer0;
	
	/* is a timer available? */
	if ( !gptimer )
		return;
	
	/* Get timer 0 */
	timer0 = &gptimer->e[1];
	
	/* Reset timer, and clear interrupt */
	timer0->ctrl = 0;
	
	/* set stop value */
	timer0->rld = ticks;
	
	/* Start timer, no interrupt*/
	timer0->ctrl = LEON3_GPTIMER_CTRL_EN | LEON3_GPTIMER_CTRL_LD;
	
	/* Wait for timer to expire */
	while( !(timer0->ctrl & LEON3_GPTIMER_CTRL_IP) ){
		;
	}
	
	/* reset timer / clear pending interrupt */
	timer0->ctrl = 0;
#endif
}

/* initiate and setup timer0 interrupt to 1MHz 
 * Return irq number for timer int or a negative number for 
 * dealing with self
 */
int timer_interrupt_init_cpu(void)
{
	/* 1ms ticks */
	gptimer->e[0].val =  0;
	gptimer->e[0].rld = 999 ;//(((1000000 / 100) - 1));
	gptimer->e[0].ctrl =  (LEON3_GPTIMER_EN | LEON3_GPTIMER_RL | LEON3_GPTIMER_LD | LEON3_GPTIMER_IRQEN);
	
	return gptimer_irq;
}

/*
 * This function is intended for SHORT delays only.
 */
unsigned long cpu_usec2ticks(unsigned long usec)
{
	/* timer set to 1kHz ==> 1 clk tick = 1 msec */
	return (usec/1000);
}

unsigned long cpu_ticks2usec(unsigned long ticks)
{
	/* 1tick = 1usec */
	return ticks*1000;
}
