/* Interface for accessing Gaisler AMBA Plug&Play Bus.
 * The AHB bus can be interfaced with a simpler APB bus,
 * also freely available in GRLIB at www.gaisler.com.
 * 
 * Copyright (C) 2007 Daniel Hellstrm, hedani3l.no.bots.here.yahoo.com
 */

#ifndef __AMBAPP_H__
#define __AMBAPP_H__

/* Default location of Plug&Play info 
 * normally 0xfffff000 for AHB masters
 * and 0xfffff800 for AHB slaves.
 * Normally no need to change this.
 */
#define LEON3_IO_AREA 0xfff00000
#define LEON3_CONF_AREA  0xff000
#define LEON3_AHB_SLAVE_CONF_AREA (1 << 11)

/* Max devices this software will support */
#define LEON3_AHB_MASTERS 8
#define LEON3_AHB_SLAVES 8
//#define LEON3_APB_MASTERS 1 /* Number of APB buses that has Plug&Play */
#define LEON3_APB_SLAVES 16 /* Total number of APB slaves per APB bus */

/* Vendor codes */
#define VENDOR_GAISLER   1
#define VENDOR_PENDER    2
#define VENDOR_ESA       4
#define VENDOR_OPENCORES 8

/* Gaisler Research device id's */
#define GAISLER_LEON3    0x003
#define GAISLER_LEON3DSU 0x004
#define GAISLER_ETHAHB   0x005
#define GAISLER_APBMST   0x006
#define GAISLER_AHBUART  0x007
#define GAISLER_SRCTRL   0x008
#define GAISLER_SDCTRL   0x009
#define GAISLER_APBUART  0x00C
#define GAISLER_IRQMP    0x00D
#define GAISLER_AHBRAM   0x00E
#define GAISLER_GPTIMER  0x011
#define GAISLER_PCITRG   0x012
#define GAISLER_PCISBRG  0x013
#define GAISLER_PCIFBRG  0x014
#define GAISLER_PCITRACE 0x015
#define GAISLER_PCIDMA   0x016
#define GAISLER_AHBTRACE 0x017
#define GAISLER_ETHDSU   0x018
#define GAISLER_PIOPORT  0x01A
#define GAISLER_AHBJTAG  0x01c
#define GAISLER_ATACTRL  0x024
#define GAISLER_VGA      0x061
#define GAISLER_KBD      0X060
#define GAISLER_ETHMAC   0x01D
#define GAISLER_GRUSB1D1 0x027
#define GAISLER_GRUSB2D0 0x028

#define GAISLER_L2TIME   0xffd	/* internal device: leon2 timer */
#define GAISLER_L2C      0xffe	/* internal device: leon2compat */
#define GAISLER_PLUGPLAY 0xfff	/* internal device: plug & play configarea */


/* European Space Agency device id's */
#define ESA_LEON2        0x2
#define ESA_MCTRL        0xF

/* Opencores device id's */
#define OPENCORES_PCIBR  0x4
#define OPENCORES_ETHMAC 0x5

/* Vendor codes */

/* 
 *
 * Macros for manipulating Configuration registers  
 *
 */

#define amba_vendor(x) (((x) >> 24) & 0xff)

#define amba_device(x) (((x) >> 12) & 0xfff)

#define amba_membar_start(mbar) \
 (((mbar) & 0xfff00000) & (((mbar) & 0xfff0) << 16))

#define amba_iobar_start(base, iobar) \
 ((base) | ((((iobar) & 0xfff00000)>>12) & (((iobar) & 0xfff0)<<4)) )

#define amba_irq(conf) ((conf) & 0xf)

#define amba_ver(conf) (((conf)>>5) & 0x1f)

#define amba_membar_type(mbar) ((mbar) & 0xf)

#define AMBA_TYPE_APBIO 0x1
#define AMBA_TYPE_MEM   0x2
#define AMBA_TYPE_AHBIO 0x3

#define AMBA_TYPE_AHBIO_ADDR(addr) (LEON3_IO_AREA | ((addr) >> 12))

#ifndef __ASSEMBLER__

#ifdef CONFIG_CMD_AMBAPP
/* AMBA Plug&Play Name of Vendors and devices */

/* Return name of device */
char *ambapp_device_id2str(int vendor, int id);

/* Return name of vendor */
char *ambapp_vendor_id2str(int vendor);
#endif

/*
 *  Types and structure used for AMBA Plug & Play bus scanning 
 */

/* AMBA Plug&Play AHB information layout */
typedef struct {
	unsigned int conf;
	unsigned int userdef[3];
	unsigned int bars[4];
} ahbctrl_pp_dev;

/* Prototypes for scanning AMBA Plug&Play bus for AMBA 
 *  i)   AHB Masters
 *  ii)  AHB Slaves 
 *  iii) APB Slaves (APB MST is a AHB Slave)
 */

typedef struct {
	unsigned char irq;
	unsigned char ver;
	unsigned int  address;
} ambapp_apbdev;

typedef struct {
	unsigned char irq;
	unsigned char ver;
	unsigned int  userdef[3];
	unsigned int  address[4];
} ambapp_ahbdev;

/* All functions return the number of found devices
 * 0 = no devices found
 */

/****************************** APB SLAVES ******************************/
int ambapp_apb_count(unsigned int vendor, unsigned int driver);
int ambapp_apb_first(unsigned int vendor, unsigned int driver, ambapp_apbdev *dev);
int ambapp_apb_next(unsigned int vendor, unsigned int driver, ambapp_apbdev *dev, int index);
int ambapp_apbs_first(unsigned int vendor, unsigned int driver, ambapp_apbdev *dev, int max_cnt);

/****************************** AHB MASTERS ******************************/
int ambapp_ahbmst_count(unsigned int vendor, unsigned int driver);
int ambapp_ahbmst_first(unsigned int vendor, unsigned int driver, ambapp_ahbdev *dev);
int ambapp_ahbmst_next(unsigned int vendor, unsigned int driver, ambapp_ahbdev *dev, int index);
int ambapp_ahbmsts_first(unsigned int vendor, unsigned int driver, ambapp_ahbdev *dev, int max_cnt);

/****************************** AHB SLAVES ******************************/
int ambapp_ahbslv_count(unsigned int vendor, unsigned int driver);
int ambapp_ahbslv_first(unsigned int vendor, unsigned int driver, ambapp_ahbdev *dev);
int ambapp_ahbslv_next(unsigned int vendor, unsigned int driver, ambapp_ahbdev *dev, int index);
int ambapp_ahbslvs_first(unsigned int vendor, unsigned int driver, ambapp_ahbdev *dev, int max_cnt);

/****************************** AHB/APB only regs functions ******************************
 * During start up, no memory is available we can use the simplified functions 
 * to get to the memory controller.
 *
 * Functions uses no stack/memory, only registers.
 */
unsigned int ambapp_apb_next_nomem(
 	register unsigned int vendor,  /* Plug&Play Vendor ID */
	register unsigned int driver,  /* Plug&Play Device ID */
	register int index);

ahbctrl_pp_dev *ambapp_ahb_next_nomem(
	register unsigned int vendor,  /* Plug&Play Vendor ID */
	register unsigned int driver,  /* Plug&Play Device ID */
	register unsigned int opts,    /* scan for AHB 1=slave, 0=masters */
	register int index);

 

#endif

#endif
